@@ -22,6 +22,7 @@
 #include <sys/stat.h>
 #include <unistd.h>
 #import "UnicodeWrapper.h"
+#import "Album.h"
 
 // number of tracks in tracklist downloaded in between progress bar updates (MTP only)
 #define TRACKLIST_PROGRESS_UPDATE_INTERVAL	10
@@ -67,6 +68,8 @@ StatusDisplayer *statusDisplayerGlobal;
 		supportedFiletypes = NULL;
 		
 		device = NULL;
+    
+    albums = nil;
 	}
 	return self;
 }
@@ -257,7 +260,7 @@ StatusDisplayer *statusDisplayerGlobal;
 	  
 		LIBMTP_Init();
 		device = LIBMTP_Get_First_Device();
-		
+    
 		if (device == NULL)
 		{
 			connected = NO;
@@ -297,16 +300,26 @@ StatusDisplayer *statusDisplayerGlobal;
 	if (mtpDevice)
 	{
 		char *tempString =  LIBMTP_Get_Modelname(device);
-		deviceString = [NSString stringWithUTF8String:tempString];
-		free(tempString);
+    if (tempString != NULL)
+    {
+      deviceString = [NSString stringWithUTF8String:tempString];
+      free(tempString);
+    }
 
 		tempString = LIBMTP_Get_Serialnumber(device);
-		deviceIDString = [NSString stringWithUTF8String:tempString];
-		free(tempString);
+		if (tempString != NULL)
+    {
+      deviceIDString = [NSString stringWithUTF8String:tempString];
+      free(tempString);
+    }
 
 		tempString = LIBMTP_Get_Deviceversion(device);
-		NSString *versionString = [NSString stringWithUTF8String:tempString];
-		free(tempString);
+    NSString *versionString = nil;
+    if (tempString != NULL)
+    {
+      versionString = [NSString stringWithUTF8String:tempString];
+      free(tempString);
+    }
 		
 		NSArray *versions = [versionString componentsSeparatedByString:@"_"];
 		if ([versions count] != 2)
@@ -328,6 +341,8 @@ StatusDisplayer *statusDisplayerGlobal;
 		}
 		
 		LIBMTP_Get_Supported_Filetypes(device, &supportedFiletypes, &supportedFiletypesLength);
+    
+    [self downloadAlbumList];
 		
 		// download the data files to the cache
 		//[self dataFiles];
@@ -369,6 +384,8 @@ StatusDisplayer *statusDisplayerGlobal;
 		cachedTrackList = nil;
 		[cachedDataFileList release];
 		cachedDataFileList = nil;
+    [albums release];
+    albums = nil;
 		
 		// send notification
 		[statusDisplayer postNotificationName:NOTIFICATION_DISCONNECTED object:self];
@@ -678,6 +695,8 @@ StatusDisplayer *statusDisplayerGlobal;
 		}
 		
 		[track setItemID:trackmeta->item_id];
+    
+    [self addTrack:track ToAlbumWithName:[track album]];
 		
 		 LIBMTP_destroy_track_t(trackmeta);
 	}
@@ -1704,6 +1723,10 @@ int mtp_progress (uint64_t const sent, uint64_t const total, void const * const
 			NSLog(@"Could not get battery level property value");
 			return 0;
 		}
+    else
+    {
+      NSLog(@"Got battery level property value");
+    }
 				
 		return value;
 	}
@@ -1829,14 +1852,140 @@ int mtp_progress (uint64_t const sent, uint64_t const total, void const * const
 {
 	if (!mtpDevice)
 		return YES;
+  
+  if (supportedFiletypesLength == 0)
+  {
+    NSLog(@"Player claims it supports no types; we will allow all");
+    return YES;
+  }
 	
+  NSLog(@"checking to see if we support type %d", type);
+  
 	int i = 0;
 	for (i = 0; i < supportedFiletypesLength; i++)
 	{
+    NSLog(@"supported type: %d", supportedFiletypes[i]);
 		if (supportedFiletypes[i] == type)
 			return YES;
 	}
 	return NO;
 }
 
+- (void)downloadAlbumList
+{
+	if (![self isConnected] || !mtpDevice)
+		return;
+	
+  if (albums != nil)
+    [albums release];
+  
+	albums = [[NSMutableArray alloc] init];
+	
+  LIBMTP_album_t *libmtpAlbums = LIBMTP_Get_Album_List(device);
+  if (libmtpAlbums == NULL)
+  {
+    // no albums or we failed
+    return;
+  }
+  LIBMTP_album_t *alb, *tmp;
+  alb = libmtpAlbums;
+  while (alb != NULL) {
+		Album *newAlbum = [[Album alloc] init];
+			
+    [newAlbum setAlbumID:alb->album_id];
+    if (alb->name != NULL)
+      [newAlbum setName:[NSString stringWithUTF8String:alb->name]];
+    int i = 0;
+    for (i = 0; i < alb->no_tracks; i++)
+    {
+      Track *newTrack = [[Track alloc] init];
+      [newTrack setItemID:alb->tracks[i]];
+      [newAlbum addTrack:newTrack];
+      [newTrack release];
+    }
+			
+    [albums addObject:newAlbum];
+    [newAlbum release];
+      
+    tmp = alb;
+    alb = alb->next;
+    LIBMTP_destroy_album_t(tmp);
+	}
+}
+
+- (void)addTrack:(Track *)track ToAlbumWithName:(NSString *)albumName
+{
+  NSLog (@"adding track with id %d to album name %@", [track itemID], albumName);
+  // don't add to a blank album
+  if ([albumName isEqualToString:@""])
+    return;
+  
+  if (albums == nil)
+    [self downloadAlbumList];
+  
+  Album *album;
+  NSEnumerator *albumEnumerator = [albums objectEnumerator];
+  
+  while (album = [albumEnumerator nextObject])
+  {
+    if ([[album name] caseInsensitiveCompare:albumName] == NSOrderedSame)
+      break;
+  }
+  
+  if (album == nil)
+  {
+    // make new album
+    album = [[Album alloc] init];
+    [album setName:albumName];
+    [album addTrack:track];
+    
+    [albums addObject:album];
+    [album release];
+  }
+  else
+  {
+    // add track to existing album
+    [album addTrack:track];
+  }
+  
+  
+  NSArray *tracks = [album tracks];
+  uint32_t size = [tracks count];
+  uint32_t *trackArray = (uint32_t*)malloc(sizeof(uint32_t)*size);
+  uint32_t i = 0;
+  for (i = 0; i < size; i++)
+    trackArray[i] = [[tracks objectAtIndex:i] itemID];
+		
+  LIBMTP_album_t *libmtpAlbum = LIBMTP_new_album_t();
+  libmtpAlbum->name = strdup([[album name] UTF8String]);
+  libmtpAlbum->album_id = [album albumID];
+  libmtpAlbum->no_tracks = size;
+  libmtpAlbum->tracks = trackArray;
+	
+  int playlistState = [album state];
+  [album setState:NJB_PL_UNCHANGED];
+		
+  if (playlistState == NJB_PL_NEW)
+  {
+    if (LIBMTP_Create_New_Album(device, libmtpAlbum, device->default_album_folder) != 0)
+    {
+      LIBMTP_destroy_album_t(libmtpAlbum);
+      NSLog(@"Could not create new album");
+      return;
+    }
+  }
+  else
+  {
+    if (LIBMTP_Update_Album(device, libmtpAlbum) != 0)
+    {
+      LIBMTP_destroy_album_t(libmtpAlbum);
+      NSLog(@"Could not update album");
+      return;
+    }
+  }
+  
+  [album setAlbumID:libmtpAlbum->album_id];
+  LIBMTP_destroy_album_t(libmtpAlbum);
+}
+
 @end
