@@ -104,15 +104,20 @@
 {
     if (validFrames == NULL)
     {
-        if ((Buffer[currentFramePosition] < 'A')||(Buffer[currentFramePosition] > 'Z')||
-            (Buffer[currentFramePosition+1] < 'A')||(Buffer[currentFramePosition+1] > 'Z') ||
-            (((Buffer[currentFramePosition+2] < 'A')||(Buffer[currentFramePosition+2] > 'Z'))&&
-            ((Buffer[currentFramePosition+2] < '0')||(Buffer[currentFramePosition+2] > '9'))))
-            {
-                framesEndAt = currentFramePosition;
-                return NO;
-            }
-        return YES;
+				if (Buffer != NULL)
+				{
+						if ((Buffer[currentFramePosition] < 'A')||(Buffer[currentFramePosition] > 'Z')||
+								(Buffer[currentFramePosition+1] < 'A')||(Buffer[currentFramePosition+1] > 'Z') ||
+								(((Buffer[currentFramePosition+2] < 'A')||(Buffer[currentFramePosition+2] > 'Z'))&&
+								 ((Buffer[currentFramePosition+2] < '0')||(Buffer[currentFramePosition+2] > '9'))))
+								{
+										framesEndAt = currentFramePosition;
+										return NO;
+								}
+						return YES;
+				}
+				else
+						return YES;
     } else    
     if ([validFrames objectForKey:[self getFrameID]] != NULL)
     {
