@@ -56,21 +56,24 @@
 /* some defines */
 /* IOUSBInterfaceInferface */
 /* 10.4 defines copied over from CVS by RAL */
-#if defined (kIOUSBInterfaceInterfaceID220)
+/* since we need to work for OS X 10.3.9 we will force to use
+ * the 10.3 versions */
+
+/*#if defined (kIOUSBInterfaceInterfaceID220)
 
 #warning "libusb being compiled for 10.4 or later"
 #define usb_interface_t IOUSBInterfaceInterface220
 #define InterfaceInterfaceID kIOUSBInterfaceInterfaceID220
 #define InterfaceVersion 220
 
-#elif defined (kIOUSBInterfaceInterfaceID197)
+#elif defined (kIOUSBInterfaceInterfaceID197)*/
 
 #warning "libusb being compiled for 10.3 or higher"
 #define usb_interface_t IOUSBInterfaceInterface197
 #define InterfaceInterfaceID kIOUSBInterfaceInterfaceID197
 #define InterfaceVersion 197
 
-#elif defined (IOUSBINTERFACE_FUNCS_190)
+/*#elif defined (IOUSBINTERFACE_FUNCS_190)
 
 #warning "libusb being compiled for 10.2 or higher"
 #define usb_interface_t IOUSBInterfaceInterface190
@@ -86,14 +89,14 @@
 
 #else
 
-/* No timeout functions available! Time to upgrade your os. */
+/ * No timeout functions available! Time to upgrade your os. * /
 #warning "libusb being compiled without support for timout bulk functions! 10.0 and up"
 #define usb_interface_t IOUSBInterfaceInterface
 #define InterfaceInterfaceID kIOUSBInterfaceInterfaceID
 #define LIBUSB_NO_TIMEOUT_INTERFACE
 #define InterfaceVersion 180
 
-#endif
+#endif */
 
 /* IOUSBDeviceInterface */
 #if defined (IOUSBDEVICE_FUNCS_197)
